@@ -11,28 +11,36 @@
 #ifndef ENV_H
 #define ENV_H
 
-extern struct ENVIRON {
+extern struct FRAME {
     int argc;
     VALUE *argv;
     ID last_func;
     struct RClass *last_class;
-    struct ENVIRON *prev;
-} *the_env;
+    struct FRAME *prev;
+    char *file;
+    int line;
+} *the_frame;
 
 extern struct SCOPE {
     struct RBasic super;
     ID *local_tbl;
     VALUE *local_vars;
-    int flags;
+    int flag;
 } *the_scope;
 
-#define SCOPE_MALLOCED (1<<0)
+#define SCOPE_ALLOCA 0
+#define SCOPE_MALLOC 1
 
 extern int rb_in_eval;
 
 extern struct RClass *the_class;
 
-#define NOEX_PUBLIC  0
-#define NOEX_PRIVATE 1
+struct RVarmap {
+    struct RBasic super;
+    ID id;
+    VALUE val;
+    struct RVarmap *next;
+};
+extern struct RVarmap *the_dyna_vars;
 
 #endif /* ENV_H */
